<?php

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

function wpc_MetaData()

{

    return array(

        'DisplayName' => 'Payment Center for Entrepreneurial Community',

        'APIVersion' => '1.1', // Use API Version 1.1

        'DisableLocalCredtCardInput' => true,

        'TokenisedStorage' => false,

    );

}

function wpc_config()

{

    return array(

        // the friendly display name for a payment gateway should be

        // defined here for backwards compatibility

        'FriendlyName' => array(

            'Type' => 'System',

            'Value' => 'PayCEC',

        ),

        // a text field type allows for single line text input

        'vendorName' => array(

            'FriendlyName' => 'PayCEC Vendor ID',

            'Type' => 'text',

            'Size' => '100',

            'Default' => '',

        ),

        // a password field type allows for masked text input

        'secretKey' => array(

            'FriendlyName' => 'PayCEC Vendor Secret Key',

            'Type' => 'text',

            'Size' => '150',

            'Default' => '',


        ),

        // the yesno field type displays a single checkbox option

        'testMode' => array(

            'FriendlyName' => 'PayCEC Test Mode',

            'Type' => 'yesno',

        ),

    );

}



function wpc_link($params)

{

    // Gateway Configuration Parameters

    $vendorName = $params['vendorName'];

    $secretKey = $params['secretKey'];

    $testMode = $params['testMode'];


    // Invoice Parameters

    $invoiceId = $params['invoiceid'];

    $amount = $params['amount'];

    $currencyCode = $params['currency'];




    // System Parameters

    $companyName = $params['companyname'];

    $systemUrl = $params['systemurl'];

    $returnUrl = $params['returnurl'];

    $langPayNow = $params['langpaynow'];

    $moduleDisplayName = $params['name'];

    $moduleName = $params['paymentmethod'];

    $whmcsVersion = $params['whmcsVersion'];

    if ($testMode =="on"){

        $wpcEndpoint = 'https://securetest.paycec.com/redirect-service';

    }else{

        $wpcEndpoint = 'https://secure.paycec.com/redirect-service';
    }

    $requestTokenEndpoint = $wpcEndpoint.'/request-token';

    $postfields =  array(
        'merchantName' => $vendorName,
        'merchantSecretKey' => $secretKey,
        'merchantToken' => time(),                                  //You can change it by your unique token builder
        'merchantReferenceCode' => 'Payment for InvoiceID #'.$invoiceId.'#'.$amount,  //Replace it by your description
        'amount' => $amount,                                           //Replace it by your total amount
        'currencyCode' => $currencyCode,                                    //Replace it by your currency code
        'returnUrl'=>$systemUrl . '/modules/gateways/callback/' . $moduleName.'_pay'. '.php',//Replace it by your return call back url
        'cancelUrl' =>$returnUrl.'&paymentfailed=true',   //Replace it by your cancel call back url
    );

    $postfields['sig'] = generateSignatureWpc($requestTokenEndpoint, $postfields, $secretKey);

    $reply = callWpc($requestTokenEndpoint, $postfields, $errorCode, $errorMessage);

    if ($reply != null && $reply->isSuccessful == 'true') {

        //Did request token successfully!
        $token = $reply->token;

        //Redirect your buyer to the Secure PayCEC Webscreen to make payment
        $redirectEndpoint = $wpcEndpoint.'/webscreen?token='.$token;

        $code = '<form method="post" action='.$redirectEndpoint.' name="frmTransaction" id="frmTransaction">	
				<input type="submit" value="Pay Now" />
				</form>';

        return $code;


    } else {
        //Error handling
        if ($reply) {
            $messageErr = '<p style="color: red">-We\'ve got some errors retrieving from PayCEC service.</p>'
                .'<p style="color: red">- Error Code: ' . $reply->errorCode . '</p>'.'<p style="color: red">- Error Message:'
                . $reply->message . '</p>';
            if (isset($reply->errorField)) {
                $messageErr .= '<p style="color: red">- Error Field: ' . $reply->errorField . '</p>';
            }
        }
        else if (!empty($errorMessage)) {
            $messageErr = '<p style="color: red">We\'ve got some errors when connecting to Secure PayCEC service.</p>'
                .'<p style="color: red">- Error Code: ' . $errorCode . '</p>'
                .'<p style="color: red">- Error Message: ' . $errorMessage . '</p>';
        }

         return $messageErr;

    }


}

function generateSignatureWpc($endpoint, $params, $secretKey)
{
    ksort($params);
    $sig = $endpoint.'?'.http_build_query($params);
    return hash_hmac('sha512', $sig, $secretKey, false);
}

/**
 * Helper function to call to Secure PayCEC service
 * @param string $endpoint
 * @param array $params
 * @param integer $errorNo
 * @param string $errorMessage
 * @return mixed|null
 */
function callWpc($endpoint, $params, &$errorNo = 0, &$errorMessage = '')
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $endpoint);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));

    $resultString = curl_exec ($ch);

    if (curl_errno($ch)) {
        $errorNo = curl_errno($ch);
        $errorMessage = curl_error($ch);
        curl_close($ch);

        return null;
    }

    curl_close($ch);

    return json_decode($resultString);
}



?>